#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "main.h"
#include "playfield_editor.h"
#include "level_ctrl.h"
#include "image_win_ctrl.h"
#include "level_properties.h"

extern Level *level;
extern Theme *theme;

void
on_mainframe_destroy_event             (GtkObject       *object,
                                        gpointer         user_data)
{
	GtkWidget *level_notebook;
	PlayFieldEditor *pfe;

	/* avoid function call through destroy process */
	level_notebook = lookup_widget(get_app(), "level_notebook");
	gtk_signal_disconnect_by_func(GTK_OBJECT(level_notebook),
				      (GFunc) on_level_switch_page, NULL);

	/* clean up */
	if(level_ctrl_are_levels_unsaved())
	{
		level_ctrl_save_all_levels();
	}

	pfe = get_level_editor();
	if(pfe)
	{
		playfield_editor_destroy(pfe);
	}
	pfe = get_goal_editor();
	if(pfe)
	{
		playfield_editor_destroy(pfe);
	}
	image_win_ctrl_destroy();
	level_ctrl_destroy();
	
	gtk_main_quit();
}


void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *app;
	app = get_app();
	g_return_if_fail(app != NULL);
	gtk_widget_destroy(app);    	
}


void
on_clear1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_properties1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

void
on_level_list_select_row               (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	Level *level;
	PlayFieldEditor *level_editor;
	PlayFieldEditor *goal_editor;

	g_print("select new row\n");

	level = level_ctrl_get_level(row);
	if(level != level_ctrl_get_actual_level())	      
	{
		/* write back properties of previous selected level */
		if(level_properties_changed())
		{
			g_print("properties changed\n");
			level_properties_update_level();

			if(level_properties_name_changed())
			{
				g_print("level renamed\n");
				level_ctrl_actual_level_renamed();
			}
			level_ctrl_set_actual_level_modified();
		}

		/* update all views with new level informations */
		level_ctrl_select_row(row);
		level_editor = get_level_editor();
		goal_editor = get_goal_editor();
		playfield_editor_set_playfield(level_editor, level->playfield);
		playfield_editor_set_playfield(goal_editor, level->goal);
		level_properties_set_level(level);
	}
#ifdef DEBUG
	else
	{
		g_print("Select row: level is the same as before.\n");
	}
#endif

}


void
on_grid_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

void
on_grid_bt_toggled                     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	PlayFieldEditor *pfe;
	gboolean state = gtk_toggle_button_get_active(togglebutton);
	
	pfe = get_level_editor();	
	playfield_editor_show_grid(pfe, state);
	pfe =  get_goal_editor();	
	playfield_editor_show_grid(pfe, state);
}


void
on_moveable_list_select_icon           (GnomeIconList   *gnomeiconlist,
                                        gint             icon,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	image_win_ctrl_select_moveable(icon);
}


void
on_connection_list_select_icon         (GnomeIconList   *gnomeiconlist,
                                        gint             icon,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	image_win_ctrl_select_connection(icon);
}


void
on_connection_list_unselect_icon       (GnomeIconList   *gnomeiconlist,
                                        gint             icon,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	image_win_ctrl_unselect_connection(icon);	
}


void
on_obstacle_list_select_icon           (GnomeIconList   *gnomeiconlist,
                                        gint             icon,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	image_win_ctrl_select_obstacle(icon);
}


void
on_new_file_activate                   (GtkWidget     *item,
                                        gpointer       user_data)
{
	GtkNotebook *level_theme_notebook;
	gint active_page;
	
	level_theme_notebook = GTK_NOTEBOOK(lookup_widget(get_app(), 
							  "level_theme_notebook"));
	/* check if the level page is visible */
	active_page = gtk_notebook_current_page(level_theme_notebook);
	if(active_page == 0 /* level page */)
	{
		GtkWidget *new_level_win;
		GtkWidget *level_name;
		GtkWidget *level_file;
		GtkWidget *theme_selector;
		GtkWidget *level_row;
		GtkWidget *level_col;
		GtkWidget *goal_row;
		GtkWidget *goal_col;
		gint button;

		new_level_win = create_new_level_win();

		/* init level name */
		level_name = lookup_widget(new_level_win, "level_name");
		gtk_entry_set_text(GTK_ENTRY(level_name), "noname");

		/* init file name */
		level_file = lookup_widget(new_level_win, "level_file");
		gtk_entry_set_text(GTK_ENTRY(level_file), "noname.xml");

		/* init theme selector */
		theme_selector = lookup_widget(new_level_win, "theme_selector");
		gtk_entry_set_text(GTK_ENTRY(theme_selector), "fabian");

		/* init level row/col */
		level_row = lookup_widget(new_level_win, "level_row");
		level_col = lookup_widget(new_level_win, "level_col");
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(level_row), 10);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(level_col), 10);

		/* init goal row/col */
		goal_row = lookup_widget(new_level_win, "goal_row");
		goal_col = lookup_widget(new_level_win, "goal_col");
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(goal_row), 10);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(goal_col), 10);

		button = gnome_dialog_run(GNOME_DIALOG(new_level_win));
	       
		if(button == 0)
		{
			gchar *name;
			gchar *file;
			gchar *theme;
			guint pf_row;
			guint pf_col;
			guint gl_row;
			guint gl_col;

			/* get values */
			name = gtk_editable_get_chars(GTK_EDITABLE(level_name),0,-1);
			file = gtk_editable_get_chars(GTK_EDITABLE(level_file),0,-1);
			theme = gtk_editable_get_chars(GTK_EDITABLE(theme_selector),0,-1);
			pf_row = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(level_row));
			pf_col = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(level_col));
			gl_row = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(goal_row));
			gl_col = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(goal_col));

			/* insert into level ctrl */
			level_ctrl_new_level(name, file, theme, pf_row, pf_col,
					     gl_row, gl_col);
		}		

		gtk_widget_destroy(GTK_WIDGET(new_level_win));
	}
	else if(active_page == 1 /* theme page */)
	{
		g_print("theme creation not supported yet.\n");
	}
}


void
on_resize_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkNotebook *level_theme_notebook;
	gint active_page;
	
	level_theme_notebook = GTK_NOTEBOOK(lookup_widget(get_app(), 
							  "level_theme_notebook"));
	/* check if the level page is visible */
	active_page = gtk_notebook_current_page(level_theme_notebook);
	if(active_page == 0 /* level page */)
	{
		GtkNotebook *level_notebook;
		PlayFieldEditor *pfe = NULL;

		level_notebook = GTK_NOTEBOOK(lookup_widget(get_app(),
							    "level_notebook"));
		/* check whether the level or goal editor is visible */
		active_page = gtk_notebook_current_page(level_notebook);
		switch(active_page)
		{
		case 0 /* level editor */:
			pfe = get_level_editor();
			break;
		case 1 /* goal editor */:
			pfe = get_goal_editor();
			break;
		case 2 /* level properties */:
			break;
		}
		
		if(pfe)
		{
			GtkWidget *window;
			GtkWidget *old_row_entry;
			GtkWidget *old_col_entry;
			GtkWidget *new_row_entry;
			GtkWidget *new_col_entry;
			gint n_rows, n_cols;
			gchar* str_buffer;
			gint button;

			/* create window */
			window = create_resize_grid_win();

			/* get entry widgets */
			old_row_entry = lookup_widget(window, "old_row");
			old_col_entry = lookup_widget(window, "old_column");
			new_row_entry = lookup_widget(window, "new_row");
			new_col_entry = lookup_widget(window, "new_column");
			
			/* set values */
			n_rows = pfe->pf->n_rows;
			n_cols = pfe->pf->n_cols;
			str_buffer = g_malloc(5*sizeof(gchar));
			g_snprintf(str_buffer, 5, "%i", n_rows);
			gtk_entry_set_text(GTK_ENTRY(old_row_entry), str_buffer);
			g_snprintf(str_buffer, 5, "%i", n_cols);
			gtk_entry_set_text(GTK_ENTRY(old_col_entry), str_buffer);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(new_row_entry),
						 n_rows);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(new_col_entry),
						 n_cols);
			g_free(str_buffer);
			
			/* show dialog */
			button = gnome_dialog_run(GNOME_DIALOG(window));

			if(button == 0)
			{
				/* user clicked ok */
				n_rows = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(new_row_entry));
				n_cols = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(new_col_entry));
				playfield_editor_resize(pfe, n_rows, n_cols);
			}
			gtk_widget_destroy(window);
		}
	}
}

void
on_save_activate                       (GtkWidget       *item,
                                        gpointer         user_data)
{
	level_ctrl_save_actual_level();
}


void
on_save_all_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	level_ctrl_save_all_levels();
}


void
on_delete_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	level_ctrl_remove_actual_level();
}


void
on_level_up_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	level_ctrl_move_up();
}


void
on_level_down_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	level_ctrl_move_down();
}


void
on_level_switch_page                   (GtkNotebook     *notebook,
                                        GtkNotebookPage *page,
                                        gint             page_num,
                                        gpointer         user_data)
{
	if(page_num == 2 /* level properties */)
	{
		level_properties_update_view();
	}
	else
	{
		if(level_properties_changed())
		{
			g_print("properties changed\n");
			level_properties_update_level();

			if(level_properties_name_changed())
			{
				level_ctrl_actual_level_renamed();
			}
			level_ctrl_set_actual_level_modified();
		}

		g_print("other page .. \n");
	}
}
