/* Atomixed -- the level-/themeeditor for atomix.
 * Copyright (C) 1999-2000 Jens Finke
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "level_ctrl.h"
#include "level.h"
#include "main.h"
#include "support.h"

/*=================================================================
 
  Declaration of global variables

  ---------------------------------------------------------------*/
Level *selected_level;     /* actual selected level */
gint selected_row;         /* actual selected row */
GSList *modified_levels;   /* list of all modified levels */

/*=================================================================
 
  Declaration of internal functions

  ---------------------------------------------------------------*/

void level_ctrl_save_level(gpointer data, gpointer user_data);

void level_ctrl_set_level_modified(Level *level);

void level_ctrl_remove_level(Level *level);

void destroy_level_item(gpointer data, gpointer user_data);

/*=================================================================
 
  Level Control initialisation and clean up

  ---------------------------------------------------------------*/
void level_ctrl_init(void)
{
	GtkWidget *app;
	GtkWidget *level_list;
	Level *level;
	gchar *text[2];
	gchar *str_buffer;
	gint length;
	gint row = 1;
	gint clist_row;
	gchar *first_level;
	
	app = get_app();
	
	level_list = GTK_WIDGET(lookup_widget(app, "level_list"));
	gtk_clist_freeze(GTK_CLIST(level_list));

	first_level = level_get_first_level();
	level = level_load_xml(first_level);

	selected_level = level;
	selected_row = 0;
	modified_levels = NULL;

	while(level != NULL)		
	{
		gchar *next;

		str_buffer = g_malloc(3*sizeof(gchar));
		length = g_snprintf(str_buffer, 3, "%i", row++);
		text[0] = str_buffer;
		text[1] = g_strdup(level->name);
		
		clist_row = gtk_clist_append(GTK_CLIST(level_list), text);
		gtk_clist_set_row_data(GTK_CLIST(level_list), clist_row, level);
		
		next = g_strdup(level->next);
		
		level = level_load_xml(next);		
	}	
	gtk_clist_select_row(GTK_CLIST(level_list), 0, 0);
	
	gtk_clist_thaw(GTK_CLIST(level_list));
}

void level_ctrl_destroy(void)
{
	Level *level;
	GtkCList *level_list;
	gint row = 0;

	level_list = GTK_CLIST(lookup_widget(get_app(), "level_list"));
	level = (Level*)gtk_clist_get_row_data(level_list, row++); 

	while(level != NULL)
	{
		level_destroy(level);
		level = (Level*)gtk_clist_get_row_data(level_list, row++); 
	}

	g_slist_free(modified_levels);
}

void destroy_level_item(gpointer data, gpointer user_data)
{
        Level *level = (Level*) data;
	level_destroy(level);
}

/*=================================================================
 
  Level Control functions

  ---------------------------------------------------------------*/

void level_ctrl_select_row(gint row)
{
	selected_level = level_ctrl_get_level(row);
	selected_row = row;
}

void level_ctrl_move_up(void)
{
	GtkWidget *level_list;
	
	g_return_if_fail(selected_level != NULL);
	
	level_list = lookup_widget(get_app(), "level_list");
	if(!selected_level->first_level)
	{
		gchar *str_buffer;
		Level *level = selected_level;
		Level *prev_level = NULL;
		Level *prev_prev_level = NULL;
		gint row;
		gint prev_row;
		gint prev_prev_row;

		row = selected_row;
		prev_row = row - 1;
		prev_prev_row = row - 2;

		g_print("Row: %i PrevRow: %i PrevPrevRow: %i\n", row, prev_row, prev_prev_row);

		prev_level =  level_ctrl_get_level(prev_row);
		if(prev_level->first_level)
		{
			prev_level->first_level = FALSE;
			level->first_level = TRUE;
			
			g_free(prev_level->next);
			prev_level->next = g_strdup(level->next);
			
			g_free(level->next);
			level->next = g_strdup(prev_level->name);

			level_ctrl_set_level_modified(level);
			level_ctrl_set_level_modified(prev_level);
		}
		else
		{
			prev_prev_level = level_ctrl_get_level(prev_prev_row);

			g_free(prev_prev_level->next);
			prev_prev_level->next = g_strdup(level->name);

			g_free(prev_level->next);
			prev_level->next = g_strdup(level->next);

			g_free(level->next);
			level->next = g_strdup(prev_level->name);

			level_ctrl_set_level_modified(level);
			level_ctrl_set_level_modified(prev_level);
			level_ctrl_set_level_modified(prev_prev_level);
		}


		/* get the right row numbering */
		str_buffer = g_malloc(3*sizeof(gchar));
		g_snprintf(str_buffer, 3, "%i",  row+1);
		gtk_clist_set_text(GTK_CLIST(level_list), prev_row, 0, str_buffer);
		g_snprintf(str_buffer, 3, "%i", prev_row+1);
		gtk_clist_set_text(GTK_CLIST(level_list), row, 0, str_buffer);

		/* swap rows */
		gtk_clist_swap_rows(GTK_CLIST(level_list), row, prev_row);

		selected_row = prev_row;
	}
}

void level_ctrl_move_down(void)
{
	GtkWidget *level_list;
	
	g_return_if_fail(selected_level != NULL);
	
	level_list = lookup_widget(get_app(), "level_list");
	if(!level_is_last_level(selected_level))
	{
		gchar *str_buffer;
		Level *level = selected_level;
		Level *prev_level = NULL;
		Level *next_level = NULL;
		gint row;
		gint prev_row;
		gint next_row;

		row = selected_row;
		prev_row = row - 1;
		next_row = row + 1;

		next_level =  level_ctrl_get_level(next_row);
		if(level->first_level)
		{
			level->first_level = FALSE;
			next_level->first_level = TRUE;
			
			g_free(level->next);
			level->next = g_strdup(next_level->next);
			
			g_free(next_level->next);
			next_level->next = g_strdup(level->name);

			level_ctrl_set_level_modified(level);
			level_ctrl_set_level_modified(next_level);
		}
		else
		{
			prev_level = level_ctrl_get_level(prev_row);

			g_free(prev_level->next);
			prev_level->next = g_strdup(next_level->name);

			g_free(level->next);
			level->next = g_strdup(next_level->next);

			g_free(next_level->next);
			next_level->next = g_strdup(level->name);

			level_ctrl_set_level_modified(level);
			level_ctrl_set_level_modified(prev_level);
			level_ctrl_set_level_modified(next_level);
		}

		/* get the right row numbering */
		str_buffer = g_malloc(3*sizeof(gchar));
		g_snprintf(str_buffer, 3, "%i",  row+1);
		gtk_clist_set_text(GTK_CLIST(level_list), next_row, 0, str_buffer);
		g_snprintf(str_buffer, 3, "%i", next_row+1);
		gtk_clist_set_text(GTK_CLIST(level_list), row, 0, str_buffer);

		/* swap rows */
		gtk_clist_swap_rows(GTK_CLIST(level_list), row, next_row);

		selected_row = next_row;
	}
}

gboolean level_ctrl_are_levels_unsaved(void)
{
	return !(g_slist_length(modified_levels) == 0);
}

void level_ctrl_save_all_levels(void)
{
	g_slist_foreach(modified_levels, (GFunc) level_ctrl_save_level, NULL);
}

/* 
 * functions which work on actual selected level 
 */

Level* level_ctrl_get_actual_level(void)
{
	return selected_level;
}

void level_ctrl_save_actual_level(void)
{
	level_ctrl_save_level((gpointer)selected_level, NULL);
}

void level_ctrl_set_actual_level_modified(void)
{
	level_ctrl_set_level_modified(selected_level);
}

void level_ctrl_remove_actual_level(void)
{
	level_ctrl_remove_level(selected_level);
}

void level_ctrl_actual_level_renamed(void)
{
	GtkWidget *level_list;
	gchar *str_buffer = NULL; 
	
	g_return_if_fail(selected_level != NULL);

	level_list = lookup_widget(get_app(), "level_list");
	
	/* change name in level list */
	if(selected_level->modified)
	{
		str_buffer = g_strconcat(selected_level->name, "*", NULL);
	}
	else
	{
		str_buffer = g_strdup(selected_level->name);
	}
	gtk_clist_set_text(GTK_CLIST(level_list), selected_row, 1,
			   str_buffer);
	g_free(str_buffer);

	/* change referenz in previous level */
	if(!selected_level->first_level)
	{
		Level *previous_level = level_ctrl_get_level(selected_row - 1);
		if(previous_level->next)
		{
			g_free(previous_level->next);
		}
		previous_level->next = g_strdup(selected_level->name);
		level_ctrl_set_level_modified(previous_level);
	}
}

/*
 * functions which work on arbitrary levels 
 */
void level_ctrl_remove_level(Level *level)
{
	GtkCList *level_list;
	gint clist_row;
	Level *next_level = NULL;
	Level *previous_level = NULL;

	level_list = GTK_CLIST(lookup_widget(get_app(), "level_list"));
	clist_row =  gtk_clist_find_row_from_data(GTK_CLIST(level_list),
						  (gpointer)level);

	/* make correct connection between levels */
	if(clist_row == 0)
	{
		if(!level_is_last_level(level))
		{
			next_level = level_ctrl_get_level(clist_row + 1);
			next_level->first_level = TRUE;
			level_ctrl_set_level_modified(next_level);
		}
	}
	else
	{
		previous_level = level_ctrl_get_level(clist_row - 1);
		g_free(previous_level->next);
		previous_level->next = g_strdup(level->next);
		level_ctrl_set_level_modified(previous_level);
	}

	/* change selection if actual level is deleted */
	if(selected_row == clist_row)
	{
		if(previous_level != NULL)
		{
			/* select previous level */
			gtk_clist_select_row(GTK_CLIST(level_list), 
					     clist_row-1, 0);
		}
		else if(next_level != NULL)
		{
			/* select next level */
			gtk_clist_select_row(GTK_CLIST(level_list), 
					     clist_row+1, 0);
		}
		else
		{
			gtk_clist_unselect_all(GTK_CLIST(level_list));
		}
	}
	
	/* remove from list */
	gtk_clist_remove(level_list, clist_row);

	/* remove from internal lists */
	modified_levels = g_slist_remove_link(modified_levels, (gpointer) level);
	
	/* destroy level */
	level_destroy(level);
}


Level* level_ctrl_get_level(gint row)
{
	GtkCList *level_list;
	Level *level;
	level_list = GTK_CLIST(lookup_widget(get_app(), "level_list"));
	level = (Level*)gtk_clist_get_row_data(level_list, row); 
	return level;
}

void level_ctrl_save_level(gpointer data, gpointer user_data)
{
	/* FIXME: check whether user directory exists */
	gchar *file;
	GtkWidget *level_list;
	gint clist_row;
	Level *level = (Level*) data;

	g_return_if_fail(level != NULL);

	file = g_strconcat(g_get_home_dir(),"/.atomix/levels/",
			   level->file_name, NULL);
	level_save_xml(level, file);
	g_free(file);

	/* remove modified mark */
	level_list = lookup_widget(get_app(), "level_list");
	clist_row =  gtk_clist_find_row_from_data(GTK_CLIST(level_list),
						  (gpointer)level);
	gtk_clist_set_text(GTK_CLIST(level_list), clist_row, 1,
			   level->name);
	
	/* remove from modified list */
	level->modified = FALSE;
	modified_levels = g_slist_remove_link(modified_levels,
					      (gpointer)level);

}

void level_ctrl_set_level_modified(Level *level)
{
	g_return_if_fail(level != NULL);

	if(!level->modified)
	{
		GtkWidget *level_list;
		gchar *str_buffer;
		gint clist_row;

		level->modified = TRUE;
		modified_levels = g_slist_append(modified_levels, 
						 (gpointer)level);

		level_list = lookup_widget(get_app(), "level_list");
		str_buffer = g_strconcat(level->name, "*", NULL);
		clist_row =  gtk_clist_find_row_from_data(GTK_CLIST(level_list),
							  (gpointer)level);
		gtk_clist_set_text(GTK_CLIST(level_list), clist_row, 1,
				   str_buffer);
		g_free(str_buffer);
	}	
}

void level_ctrl_new_level(gchar *name, gchar *file, gchar *theme_name,
			  guint n_rows, guint n_cols,
			  guint goal_n_rows, guint goal_n_cols)
{
	Level *level;
	GtkWidget *level_list;
	gchar *str_buffer;
	gchar *text[2];
	gint clist_row;

	g_return_if_fail(name!=NULL);
	g_return_if_fail(file!=NULL);
	g_return_if_fail(theme_name!=NULL);

	/* init level struct */
	level = level_new();
	level->name = name;
	level->theme_name = theme_name;
	level->file_name = file;
	level->playfield = playfield_new();
	level->modified = TRUE;
	playfield_set_matrix_size(level->playfield, n_rows, n_cols);
	level->goal = playfield_new();
	playfield_set_matrix_size(level->goal, goal_n_rows, goal_n_cols);

	/* add it to the list */
	level_list = lookup_widget(get_app(), "level_list");	
	str_buffer = g_strconcat(level->name, "*", NULL);
	text[0] = "";
	text[1] = str_buffer;
	clist_row = gtk_clist_append(GTK_CLIST(level_list), text);
	g_free(str_buffer);

	str_buffer = g_malloc(3*sizeof(gchar));
	g_snprintf(str_buffer, 3, "%i", clist_row+1);
	gtk_clist_set_text(GTK_CLIST(level_list), clist_row, 0, str_buffer);
	g_free(str_buffer);
	gtk_clist_set_row_data(GTK_CLIST(level_list), clist_row, level);

	/* make correct connection between levels */
	if(clist_row == 0)
	{
		Level *next_level = level_ctrl_get_level(clist_row + 1);
		level->first_level = TRUE;

		if(next_level != NULL)
		{
			next_level->first_level = FALSE;
			level->next = g_strdup(next_level->name);
			level_ctrl_set_level_modified(next_level);
		}
		else
		{
			level_set_last_level(level);
		}
	}
	else
	{
		Level *previous_level = level_ctrl_get_level(clist_row - 1);
		level->next = previous_level->next;
		previous_level->next = g_strdup(level->name);
		level_ctrl_set_level_modified(previous_level);
	}


	/* select this level */
	gtk_clist_select_row(GTK_CLIST(level_list), clist_row, 0);

}

