/* Atomix -- a little mind game about atoms and molecules.
 * Copyright (C) 1999 Jens Finke
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "board.h"
#include "theme.h"
#include "level.h"
#include "main.h"
#include "gtk-time-limit.h"

void
on_new_game_activate(GtkMenuItem     *menuitem,
                     gpointer         user_data)
{
	game_new();
}

void
on_pause_game_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	game_pause();

}
void
on_continue_game_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	game_continue();
}

void
on_undo_move_activate(GtkMenuItem     *menuitem,
                      gpointer         user_data)
{
	game_undo_move();
}


void
on_score_activate(GtkMenuItem     *menuitem,
		  gpointer         user_data)
{
	gnome_scores_display("Atomix", "atomix", NULL, 0);	
}


void
on_exit_activate(GtkMenuItem     *menuitem,
                 gpointer         user_data)
{
	GtkWidget* app;

	app = get_app();
	g_return_if_fail(app != NULL);
	gtk_widget_destroy(app);    
}


void
on_preferences_activate(GtkMenuItem     *menuitem,
                        gpointer         user_data)
{
	GtkWidget* dlg;
	dlg = create_dlg_not_implemented();
	gtk_widget_show(dlg);
}


void
on_about_activate(GtkMenuItem     *menuitem,
                  gpointer         user_data)
{
	GtkWidget* dlg;
	dlg = create_dlg_about();
	gtk_widget_show(dlg);
}


gboolean
on_app_destroy_event(GtkWidget       *widget,
                     GdkEvent        *event,
                     gpointer         user_data)
{
	board_destroy();
    
	if(get_actual_level())
	{
		level_destroy(get_actual_level());
	}
	if(get_actual_theme())
	{
		theme_destroy(get_actual_theme());
	}
	if(get_preferences())
	{
		g_free(get_preferences());
	}
	level_destroy_hash_table();
	theme_destroy_hash_table();
	gtk_main_quit();

	return TRUE;
}


/* This function ist used to remove all canvas items 
   (look at board_clear())*/
void
destroy_item(gpointer ptr, gpointer data)
{
	GtkObject* obj;

	obj = GTK_OBJECT (ptr);
	gtk_object_destroy(obj);
}


/* just to create my selfmade widget gtk_time_limit */
GtkWidget*
create_time_limit_widget (gchar *widget_name, gchar *string1, gchar *string2,
			  gint int1, gint int2)
{
	return gtk_time_limit_new();
}

